﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Olap;
using Infragistics.Olap.FlatData;
using Infragistics.Controls.Grids;
using System.Collections.ObjectModel;
using System.Threading;
using System.ComponentModel;

namespace PivotGridTest
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window, INotifyPropertyChanged
    {
        private ObservableCollection<Person> _people;
        public ObservableCollection<Person> People
        {
            get { return _people; }
            set
            {
                if (_people != value)
                {
                    _people = value;
                    RaisePropertyChanged("People");
                }
            }
        }

        private string _SelectedGender;
        public string SelectedGender
        {
            get { return _SelectedGender; }
            set
            {
                if (_SelectedGender != value)
                {
                    _SelectedGender = value;
                    RaisePropertyChanged("SelectedGender");
                }
            }
        }

        private string _SelectedTypeCode;
        public string SelectedTypeCode
        {
            get { return _SelectedTypeCode; }
            set
            {
                if (_SelectedTypeCode != value)
                {
                    _SelectedTypeCode = value;
                    RaisePropertyChanged("SelectedTypeCode");
                }
            }
        }

        private bool UsePctTotalMeasure;
        protected XAMPivotFilterHelper<Person> FilterHelper;
        private ManualResetEvent _mre;

        public MainWindow()
        {
            InitializeComponent();
            DataContext = this;
            People = new ObservableCollection<Person>();
            Loaded += new RoutedEventHandler(MainWindow_Loaded);
        }

        private void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            Loaded -= MainWindow_Loaded;
            PeopleGrid.DataSource = (IOlapViewModel)FindResource("PersonFlatDataSource");
            PeopleGrid.DataSource.ResultChanged += new EventHandler<AsyncCompletedEventArgs>(DataSource_ResultChanged);
            FilterHelper = new XAMPivotFilterHelper<Person>{FlatDataSource = ((FlatDataSource)PeopleGrid.DataSource)};

            // Get data
            SelectedTypeCode = "All";
            SelectedGender = "All";
            FilterData();
            
            // Build dropdowns
            List<string> genders = People.GroupBy(x => x.Gender).Select(x => x.Key).OrderBy(x => x).ToList();
            genders.Insert(0,"All");
            GenderCombo.ItemsSource = genders;

            List<string> typeCodes = People.GroupBy(x => x.TypeCode).Select(x => x.Key).OrderBy(x => x).ToList();
            typeCodes.Insert(0, "All");
            TypeCodeCombo.ItemsSource = typeCodes;
        }
        
        private void PeopleGrid_Loaded(object sender, RoutedEventArgs e)
        {
            PeopleGrid.Loaded -= PeopleGrid_Loaded;
            
            // Set datasource on grid
            FilterHelper.FlatDataSource.ItemsSource = People;


            /* hide some filters of the XamPivotGrid which are set in code */
            PeopleGrid.MeasuresDropControl.Visibility = Visibility.Collapsed;
            PeopleGrid.ColumnsDropControl.Visibility = Visibility.Collapsed;
            PeopleGrid.FiltersDropControl.Visibility = Visibility.Collapsed;
            PeopleGrid.RowsDropControl.Visibility = Visibility.Collapsed;
            
            

            ExpandDimensions();
            
        }

        void DataSource_ResultChanged(object sender, AsyncCompletedEventArgs e)
        {
            PeopleGrid.DataSource.ResultChanged -= DataSource_ResultChanged;
            BackgroundWorker worker = new BackgroundWorker();
            worker.DoWork += new DoWorkEventHandler((a, b) =>  this.Dispatcher.Invoke(new Action(() => FilterData())));
            
            // Attempting to re-wire the ResultChanged event causes a crash:
            worker.RunWorkerCompleted += new RunWorkerCompletedEventHandler((c, d) => PeopleGrid.DataSource.ResultChanged += new EventHandler<AsyncCompletedEventArgs>(DataSource_ResultChanged));
            worker.RunWorkerAsync();
        }


        private void RefreshButton_Click(object sender, RoutedEventArgs e)
        {
            PeopleGrid.DataSource.RefreshGrid();  // Raises ResultChanged event on pivot grid, which calls FilterData
        }

        /// <summary>
        /// Change the data source to so we can change the measure to show income or income as percent of total.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ToggleMeasureButton_Click(object sender, RoutedEventArgs e)
        {

            UsePctTotalMeasure = !UsePctTotalMeasure;
            ((Button)sender).Content = "Toggle Measure" + (UsePctTotalMeasure ? " (Currently: Percentages)" : " (Currently: Income)");

            if (UsePctTotalMeasure)
                PeopleGrid.DataSource = (IOlapViewModel)FindResource("PersonFlatDataSourcePct");
            else
                PeopleGrid.DataSource = (IOlapViewModel)FindResource("PersonFlatDataSource");

            ExpandDimensions();
        }

        /// <summary>
        /// Add a measure to the pct flat data
        /// </summary>
        private void BuildPctDataSource()
        {
            
            FlatDataSource flatDataPct = (FlatDataSource)FindResource("PersonFlatDataSourcePct");
            CubeMetadata cubeMetaData = new CubeMetadata
            {
                DisplayName = "People",
                DataTypeFullName = typeof(Person).FullName
            };
            
            flatDataPct.CubesSettings.Add(cubeMetaData);

            DimensionMetadata pctIncomeMetaData = new DimensionMetadata
            {
                SourcePropertyName = "Income",
                DisplayName = "Inc. Pct Total",
                DisplayFormat = "{0:00.##%}",
                DimensionType = DimensionType.Measure,
                Aggregator = new PctTotalAggregator(flatDataPct, "ReferenceIncome")
            };
            cubeMetaData.DimensionSettings.Add(pctIncomeMetaData);

            PeopleGrid.DataSource = flatDataPct;
        }

        /// <summary>
        /// Expands the dimensions of the pivotgrid
        /// </summary>
        private void ExpandDimensions()
        {
            _mre = new ManualResetEvent(false);
            PeopleGrid.DataSource.DeferredLayoutUpdate = true;
            

            // process the expansion state
            IFilterViewModel fvm = FilterHelper.GetFilterViewModel("Gender");
            fvm.LoadFilterMembersCompleted += (s, f) => _mre.Set();

            foreach (Infragistics.Olap.Data.IFilterMember member in fvm.FilterMembers)
                FilterHelper.ExpandHierarchy(member, true);

            fvm = FilterHelper.GetFilterViewModel("AgeGroup");
            fvm.LoadFilterMembersCompleted += (s, f) => _mre.Set();

            foreach (Infragistics.Olap.Data.IFilterMember member in fvm.FilterMembers)
                FilterHelper.ExpandHierarchy(member, true);

            PeopleGrid.DataSource.DeferredLayoutUpdate = false;
        }

        private void PeopleGrid_LayoutLoaded(object sender, EventArgs e)
        {
            //if (! UsePctTotalMeasure)
            //    return; // only hide totals if we are showing percents

            //foreach (PivotDataColumn c in PeopleGrid.DataColumns.Where(x => x.IsTotal))
            //    c.IsVisible = false;

            
        }

        /// <summary>
        /// Filters the datasource based on the selected dropdown values
        /// </summary>
        protected void FilterData()
        {
            IQueryable<Person> query = Data.GetPeople();
            query = query.Where(x => SelectedGender == "All" || x.Gender == SelectedGender);
            query = query.Where(x => SelectedTypeCode == "All" || x.TypeCode == SelectedTypeCode);
            //People = new ObservableCollection<Person>(query.OrderBy(x => x.AgeGroup).ThenBy(x => x.Gender).ThenBy(x => x.TypeCode));
            People.Clear();
            
            foreach (Person p in query)
                People.Add(p);
        }

        private void GenderCombo_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            PeopleGrid.DataSource.RefreshGrid();  // Raises ResultChanged event on pivot grid, which calls FilterData
        }

        private void TypeCodeCombo_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            PeopleGrid.DataSource.RefreshGrid();  // Raises ResultChanged event on pivot grid, which calls FilterData
        }

        #region INotifyPropertyChanged members
        public event PropertyChangedEventHandler PropertyChanged;

        public void RaisePropertyChanged(string propertyName)
        {
            if(PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion
    }

    

}
